@@ -228,7 +228,7 @@ send_control_channel_string (struct context *c, const char *str, int msglevel)
     bool stat;
 
     /* buffered cleartext write onto TLS control channel */
-    stat = tls_send_payload (c->c2.tls_multi, str, strlen (str) + 1);
+    stat = tls_send_payload (c->c2.tls_multi, (uint8_t*) str, strlen (str) + 1);
 
     /* reschedule tls_multi_process */
     interval_action (&c->c2.tmp_int);
@@ -397,7 +397,7 @@ encrypt_sign (struct context *c, bool comp_frag)
    */
   if (c->c2.tls_multi)
     {
-      //tls_mutex_lock (c->c2.tls_multi);
+      /*tls_mutex_lock (c->c2.tls_multi);*/
       tls_pre_encrypt (c->c2.tls_multi, &c->c2.buf, &c->c2.crypto_options);
     }
 #endif
@@ -425,7 +425,7 @@ encrypt_sign (struct context *c, bool comp_frag)
   if (c->c2.tls_multi)
     {
       tls_post_encrypt (c->c2.tls_multi, &c->c2.buf);
-      //tls_mutex_unlock (c->c2.tls_multi);
+      /*tls_mutex_unlock (c->c2.tls_multi);*/
     }
 #endif
 #endif
@@ -705,7 +705,7 @@ process_incoming_link (struct context *c)
 	   * will load crypto_options with the correct encryption key
 	   * and return false.
 	   */
-	  //tls_mutex_lock (c->c2.tls_multi);
+	  /*tls_mutex_lock (c->c2.tls_multi);*/
 	  if (tls_pre_decrypt (c->c2.tls_multi, &c->c2.from, &c->c2.buf, &c->c2.crypto_options))
 	    {
 	      interval_action (&c->c2.tmp_int);
@@ -731,7 +731,7 @@ process_incoming_link (struct context *c)
 #ifdef USE_SSL
       if (c->c2.tls_multi)
 	{
-	  //tls_mutex_unlock (c->c2.tls_multi);
+	  /*tls_mutex_unlock (c->c2.tls_multi);*/
 	}
 #endif
       
