@@ -50,6 +50,8 @@
 #define CF_INIT_TLS_MULTI           (1<<1)
 #define CF_INIT_TLS_AUTH_STANDALONE (1<<2)
 
+static void do_init_first_time (struct context *c);
+
 void
 context_clear (struct context *c)
 {
@@ -356,6 +358,8 @@ possibly_become_daemon (const struct options *options, const bool first_time)
       ASSERT (!options->inetd);
       if (daemon (options->cd_dir != NULL, options->log) < 0)
 	msg (M_ERR, "daemon() failed");
+      if (options->log)
+	set_std_files_to_null (true);
       ret = true;
     }
   return ret;
@@ -573,7 +577,11 @@ initialization_sequence_completed (struct context *c, const unsigned int flags)
 
   /* Test if errors */
   if (flags & ISC_ERRORS)
+#ifdef WIN32
+    msg (M_INFO, "%s With Errors ( see http://openvpn.net/faq.html#dhcpclientserv )", message);
+#else
     msg (M_INFO, "%s With Errors", message);
+#endif
   else
     msg (M_INFO, "%s", message);
 
@@ -968,8 +976,19 @@ do_deferred_options (struct context *c, const unsigned int found)
       do_init_timers (c, true);
       msg (D_PUSH, "OPTIONS IMPORT: timers and/or timeouts modified");
     }
+
+#ifdef ENABLE_OCC
   if (found & OPT_P_EXPLICIT_NOTIFY)
-    msg (D_PUSH, "OPTIONS IMPORT: explicit notify parm(s) modified");
+    {
+      if (c->options.proto != PROTO_UDPv4 && c->options.explicit_exit_notification)
+	{
+	  msg (D_PUSH, "OPTIONS IMPORT: --explicit-exit-notify can only be used with --proto udp");
+	  c->options.explicit_exit_notification = 0;
+	}
+      else
+	msg (D_PUSH, "OPTIONS IMPORT: explicit notify parm(s) modified");
+    }
+#endif
 
   if (found & OPT_P_SHAPER)
     {
@@ -993,11 +1012,16 @@ do_deferred_options (struct context *c, const unsigned int found)
  * Possible hold on initialization
  */
 static bool
-do_hold (void)
+do_hold (struct context *c)
 {
 #ifdef ENABLE_MANAGEMENT
   if (management)
     {
+      /* if c is defined, daemonize before hold */
+      if (c && c->options.daemon && management_would_hold (management))
+	do_init_first_time (c);
+
+      /* block until management hold is released */
       if (management_hold (management))
 	return true;
     }
@@ -1009,7 +1033,7 @@ do_hold (void)
  * Sleep before restart.
  */
 static void
-socket_restart_pause (const struct context *c)
+socket_restart_pause (struct context *c)
 {
   bool proxy = false;
   int sec = 2;
@@ -1042,7 +1066,12 @@ socket_restart_pause (const struct context *c)
     sec = 0;
 #endif
 
-  if (do_hold ())
+#if P2MP
+  if (auth_retry_get () == AR_NOINTERACT)
+    sec = 10;
+#endif
+
+  if (do_hold (NULL))
     sec = 0;
 
   if (sec)
@@ -1061,7 +1090,7 @@ do_startup_pause (struct context *c)
   if (!c->first_time)
     socket_restart_pause (c);
   else
-    do_hold ();
+    do_hold (NULL);
 }
 
 /*
@@ -1226,6 +1255,28 @@ do_init_crypto_tls_c1 (struct context *c)
        * SSL context.
        */
       c->c1.ks.ssl_ctx = init_ssl (options);
+      if (!c->c1.ks.ssl_ctx)
+	{
+#if P2MP
+	  switch (auth_retry_get ())
+	    {
+	    case AR_NONE:
+	      msg (M_FATAL, "Error: private key password verification failed");
+	      break;
+	    case AR_INTERACT:
+	      ssl_purge_auth ();
+	    case AR_NOINTERACT:
+	      c->sig->signal_received = SIGUSR1; /* SOFT-SIGUSR1 -- Password failure error */
+	      break;
+	    default:
+	      ASSERT (0);
+	    }
+	  c->sig->signal_text = "private-key-password-failure";
+	  return;
+#else
+	  msg (M_FATAL, "Error: private key password verification failed");
+#endif
+	}
 
       /* Get cipher & hash algorithms */
       init_key_type (&c->c1.ks.key_type, options->ciphername,
@@ -1262,6 +1313,8 @@ do_init_crypto_tls (struct context *c, const unsigned int flags)
 
   /* initialize persistent component */
   do_init_crypto_tls_c1 (c);
+  if (IS_SIG (c))
+    return;
 
   /* Sanity check on IV, sequence number, and cipher mode options */
   check_replay_iv_consistency (&c->c1.ks.key_type, options->replay,
@@ -1492,7 +1545,7 @@ do_option_warnings (struct context *c)
 {
   const struct options *o = &c->options;
 
-#if 1 // JYFIXME -- port warning
+#if 1 /* JYFIXME -- port warning */
   if (!o->port_option_used && (o->local_port == OPENVPN_PORT && o->remote_port == OPENVPN_PORT))
     msg (M_WARN, "IMPORTANT: OpenVPN's default port number is now %d, based on an official port number assignment by IANA.  OpenVPN 2.0-beta16 and earlier used 5000 as the default port.",
 	 OPENVPN_PORT);
@@ -1724,11 +1777,11 @@ do_compute_occ_strings (struct context *c)
 #ifdef USE_CRYPTO
   msg (D_SHOW_OCC_HASH, "Local Options hash (VER=%s): '%s'",
        options_string_version (c->c2.options_string_local, &gc),
-       md5sum (c->c2.options_string_local,
+       md5sum ((uint8_t*)c->c2.options_string_local,
 	       strlen (c->c2.options_string_local), 9, &gc));
   msg (D_SHOW_OCC_HASH, "Expected Remote Options hash (VER=%s): '%s'",
        options_string_version (c->c2.options_string_remote, &gc),
-       md5sum (c->c2.options_string_remote,
+       md5sum ((uint8_t*)c->c2.options_string_remote,
 	       strlen (c->c2.options_string_remote), 9, &gc));
 #endif
 
@@ -1751,7 +1804,7 @@ do_compute_occ_strings (struct context *c)
 static void
 do_init_first_time (struct context *c)
 {
-  if (c->first_time)
+  if (c->first_time && !c->c2.did_we_daemonize)
     {
       /* get user and/or group that we want to setuid/setgid to */
       c->c2.uid_gid_specified =
@@ -2141,7 +2194,7 @@ open_management (struct context *c)
 	    }
 
 	  /* possible wait */
-	  do_hold ();
+	  do_hold (c);
 	  if (IS_SIG (c))
 	    {
 	      msg (M_WARN, "Signal received from management interface, exiting");
@@ -2297,6 +2350,8 @@ init_instance (struct context *c, const struct env_set *env, const unsigned int
     else if (child)
       crypto_flags = CF_INIT_TLS_MULTI;
     do_init_crypto (c, crypto_flags);
+    if (IS_SIG (c))
+      goto sig;
   }
 
 #ifdef USE_LZO
